@@ -17,7 +17,7 @@ use self::npm::{
 };
 use crate::command::build::{BuildProfile, Target};
 use crate::PBAR;
-use cargo_metadata::Metadata;
+use cargo_metadata::{CrateType, Metadata, TargetKind};
 use chrono::offset;
 use chrono::DateTime;
 use serde::{self, Deserialize};
@@ -30,7 +30,7 @@ use toml;
 
 const WASM_PACK_METADATA_KEY: &str = "package.metadata.wasm-pack";
 const WASM_PACK_VERSION: Option<&'static str> = option_env!("CARGO_PKG_VERSION");
-const WASM_PACK_REPO_URL: &str = "https://github.com/rustwasm/wasm-pack";
+const WASM_PACK_REPO_URL: &str = "https://github.com/drager/wasm-pack";
 
 /// Store for metadata learned about a crate
 pub struct CrateData {
@@ -85,6 +85,12 @@ struct CargoWasmPackProfiles {
         deserialize_with = "CargoWasmPackProfile::deserialize_profiling"
     )]
     profiling: CargoWasmPackProfile,
+
+    #[serde(
+        default = "CargoWasmPackProfile::default_custom",
+        deserialize_with = "CargoWasmPackProfile::deserialize_custom"
+    )]
+    custom: CargoWasmPackProfile,
 }
 
 impl Default for CargoWasmPackProfiles {
@@ -93,6 +99,7 @@ impl Default for CargoWasmPackProfiles {
             dev: CargoWasmPackProfile::default_dev(),
             release: CargoWasmPackProfile::default_release(),
             profiling: CargoWasmPackProfile::default_profiling(),
+            custom: CargoWasmPackProfile::default_custom(),
         }
     }
 }
@@ -120,6 +127,9 @@ struct CargoWasmPackProfileWasmBindgen {
 
     #[serde(default, rename = "omit-default-module-path")]
     omit_default_module_path: Option<bool>,
+
+    #[serde(default, rename = "split-linked-modules")]
+    split_linked_modules: Option<bool>,
 }
 
 /// Struct for storing information received from crates.io
@@ -276,6 +286,7 @@ impl CargoWasmPackProfile {
                 demangle_name_section: Some(true),
                 dwarf_debug_info: Some(false),
                 omit_default_module_path: Some(false),
+                split_linked_modules: Some(false),
             },
             wasm_opt: None,
         }
@@ -288,6 +299,7 @@ impl CargoWasmPackProfile {
                 demangle_name_section: Some(true),
                 dwarf_debug_info: Some(false),
                 omit_default_module_path: Some(false),
+                split_linked_modules: Some(false),
             },
             wasm_opt: Some(CargoWasmPackProfileWasmOpt::Enabled(true)),
         }
@@ -300,6 +312,20 @@ impl CargoWasmPackProfile {
                 demangle_name_section: Some(true),
                 dwarf_debug_info: Some(false),
                 omit_default_module_path: Some(false),
+                split_linked_modules: Some(false),
+            },
+            wasm_opt: Some(CargoWasmPackProfileWasmOpt::Enabled(true)),
+        }
+    }
+
+    fn default_custom() -> Self {
+        CargoWasmPackProfile {
+            wasm_bindgen: CargoWasmPackProfileWasmBindgen {
+                debug_js_glue: Some(false),
+                demangle_name_section: Some(true),
+                dwarf_debug_info: Some(false),
+                omit_default_module_path: Some(false),
+                split_linked_modules: Some(false),
             },
             wasm_opt: Some(CargoWasmPackProfileWasmOpt::Enabled(true)),
         }
@@ -332,6 +358,15 @@ impl CargoWasmPackProfile {
         Ok(profile)
     }
 
+    fn deserialize_custom<'de, D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: serde::Deserializer<'de>,
+    {
+        let mut profile = <Option<Self>>::deserialize(deserializer)?.unwrap_or_default();
+        profile.update_with_defaults(&Self::default_custom());
+        Ok(profile)
+    }
+
     fn update_with_defaults(&mut self, defaults: &Self) {
         macro_rules! d {
             ( $( $path:ident ).* ) => {
@@ -342,6 +377,7 @@ impl CargoWasmPackProfile {
         d!(wasm_bindgen.demangle_name_section);
         d!(wasm_bindgen.dwarf_debug_info);
         d!(wasm_bindgen.omit_default_module_path);
+        d!(wasm_bindgen.split_linked_modules);
 
         if self.wasm_opt.is_none() {
             self.wasm_opt = defaults.wasm_opt.clone();
@@ -368,6 +404,11 @@ impl CargoWasmPackProfile {
         self.wasm_bindgen.omit_default_module_path.unwrap()
     }
 
+    /// Get this profile's configured `[wasm-bindgen.split-linked-modules]` value.
+    pub fn wasm_bindgen_split_linked_modules(&self) -> bool {
+        self.wasm_bindgen.split_linked_modules.unwrap()
+    }
+
     /// Get this profile's configured arguments for `wasm-opt`, if enabled.
     pub fn wasm_opt_args(&self) -> Option<Vec<String>> {
         match self.wasm_opt.as_ref()? {
@@ -388,7 +429,7 @@ struct NpmData {
 }
 
 #[doc(hidden)]
-pub struct ManifestAndUnsedKeys {
+pub struct ManifestAndUnusedKeys {
     pub manifest: CargoManifest,
     pub unused_keys: BTreeSet<String>,
 }
@@ -441,16 +482,21 @@ impl CrateData {
     }
 
     /// Read the `manifest_path` file and deserializes it using the toml Deserializer.
-    /// Returns a Result containing `ManifestAndUnsedKeys` which contains `CargoManifest`
+    /// Returns a Result containing `ManifestAndUnusedKeys` which contains `CargoManifest`
     /// and a `BTreeSet<String>` containing the unused keys from the parsed file.
     ///
     /// # Errors
     /// Will return Err if the file (manifest_path) couldn't be read or
     /// if deserialize to `CargoManifest` fails.
-    pub fn parse_crate_data(manifest_path: &Path) -> Result<ManifestAndUnsedKeys> {
+    pub fn parse_crate_data(manifest_path: &Path) -> Result<ManifestAndUnusedKeys> {
         let manifest = fs::read_to_string(&manifest_path)
             .with_context(|| anyhow!("failed to read: {}", manifest_path.display()))?;
-        let manifest = toml::Deserializer::new(&manifest);
+        let manifest = toml::Deserializer::parse(&manifest).with_context(|| {
+            anyhow!(
+                "failed to create TOML deserializer for: {}",
+                manifest_path.display()
+            )
+        })?;
 
         let mut unused_keys = BTreeSet::new();
         let levenshtein_threshold = 1;
@@ -467,7 +513,7 @@ impl CrateData {
         })
         .with_context(|| anyhow!("failed to parse manifest: {}", manifest_path.display()))?;
 
-        Ok(ManifestAndUnsedKeys {
+        Ok(ManifestAndUnusedKeys {
             manifest,
             unused_keys,
         })
@@ -475,7 +521,7 @@ impl CrateData {
 
     /// Iterating through all the passed `unused_keys` and output
     /// a warning for each unknown key.
-    pub fn warn_for_unused_keys(manifest_and_keys: &ManifestAndUnsedKeys) {
+    pub fn warn_for_unused_keys(manifest_and_keys: &ManifestAndUnusedKeys) {
         manifest_and_keys.unused_keys.iter().for_each(|path| {
             PBAR.warn(&format!(
                 "\"{}\" is an unknown key and will be ignored. Please check your Cargo.toml.",
@@ -490,6 +536,7 @@ impl CrateData {
             BuildProfile::Dev => &self.manifest.package.metadata.wasm_pack.profile.dev,
             BuildProfile::Profiling => &self.manifest.package.metadata.wasm_pack.profile.profiling,
             BuildProfile::Release => &self.manifest.package.metadata.wasm_pack.profile.release,
+            BuildProfile::Custom(_) => &self.manifest.package.metadata.wasm_pack.profile.custom,
         }
     }
 
@@ -504,8 +551,8 @@ impl CrateData {
         let any_cdylib = pkg
             .targets
             .iter()
-            .filter(|target| target.kind.iter().any(|k| k == "cdylib"))
-            .any(|target| target.crate_types.iter().any(|s| s == "cdylib"));
+            .filter(|target| target.kind.iter().any(|k| k == &TargetKind::CDyLib))
+            .any(|target| target.crate_types.iter().any(|s| s == &CrateType::CDyLib));
         if any_cdylib {
             return Ok(());
         }
@@ -527,7 +574,7 @@ impl CrateData {
         match pkg
             .targets
             .iter()
-            .find(|t| t.kind.iter().any(|k| k == "cdylib"))
+            .find(|t| t.kind.iter().any(|k| k == &TargetKind::CDyLib))
         {
             Some(lib) => lib.name.replace("-", "_"),
             None => pkg.name.replace("-", "_"),
@@ -630,7 +677,7 @@ impl CrateData {
         let pkg = &self.data.packages[self.current_idx];
         let npm_name = match scope {
             Some(s) => format!("@{}/{}", s, pkg.name),
-            None => pkg.name.clone(),
+            None => pkg.name.to_string(),
         };
 
         let dts_file = if !disable_dts {
